# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_dict import remove_none_from_dict
from ..commons.types.user_id import UserId
from .types.api_token_response import ApiTokenResponse
from .types.exchange_token_response import ExchangeTokenResponse
from .types.list_api_tokens_response import ListApiTokensResponse
from .types.list_users_response import ListUsersResponse
from .types.user_response import UserResponse

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class UsersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(self, *, email: typing.Optional[str] = None) -> ListUsersResponse:
        """
        Gets a list of users

        Parameters:
            - email: typing.Optional[str]. Email of guest to return
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "users"),
            params=remove_none_from_dict({"email": email}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListUsersResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, user_id: UserId) -> UserResponse:
        """
        Gets a user

        Parameters:
            - user_id: UserId. The user id
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"users/{user_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(UserResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def list_api_tokens(
        self,
        user_id: UserId,
        *,
        tenant_id: str,
        page_size: typing.Optional[int] = None,
        page_number: typing.Optional[int] = None,
    ) -> ListApiTokensResponse:
        """
        Gets all the api tokens for a user.

        Parameters:
            - user_id: UserId. The user id

            - tenant_id: str.

            - page_size: typing.Optional[int]. Number of tokens to return in a page (default 10)

            - page_number: typing.Optional[int]. Based on pageSize, which page of records to return
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"users/{user_id}/api-token"),
            params=remove_none_from_dict({"tenantId": tenant_id, "pageSize": page_size, "pageNumber": page_number}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListApiTokensResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_api_token(self, user_id: UserId, *, tenant_id: str) -> ApiTokenResponse:
        """
        Creates an api token for authenticating against Flatfile APIs.

        Parameters:
            - user_id: UserId. The user id

            - tenant_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"users/{user_id}/api-token"),
            params=remove_none_from_dict({"tenantId": tenant_id}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ApiTokenResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def exchange_token(
        self,
        *,
        token: typing.Optional[str] = OMIT,
        email: typing.Optional[str] = OMIT,
        space_id: typing.Optional[str] = OMIT,
    ) -> ExchangeTokenResponse:
        """
        Exchange an invitation for an access token

        Parameters:
            - token: typing.Optional[str]. The token to use for the request

            - email: typing.Optional[str]. The invited guest's email, required if no token

            - space_id: typing.Optional[str]. The spaceId, required if no token
        """
        _request: typing.Dict[str, typing.Any] = {}
        if token is not OMIT:
            _request["token"] = token
        if email is not OMIT:
            _request["email"] = email
        if space_id is not OMIT:
            _request["spaceId"] = space_id
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "invitations/exchange"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ExchangeTokenResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncUsersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(self, *, email: typing.Optional[str] = None) -> ListUsersResponse:
        """
        Gets a list of users

        Parameters:
            - email: typing.Optional[str]. Email of guest to return
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "users"),
            params=remove_none_from_dict({"email": email}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListUsersResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, user_id: UserId) -> UserResponse:
        """
        Gets a user

        Parameters:
            - user_id: UserId. The user id
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"users/{user_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(UserResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def list_api_tokens(
        self,
        user_id: UserId,
        *,
        tenant_id: str,
        page_size: typing.Optional[int] = None,
        page_number: typing.Optional[int] = None,
    ) -> ListApiTokensResponse:
        """
        Gets all the api tokens for a user.

        Parameters:
            - user_id: UserId. The user id

            - tenant_id: str.

            - page_size: typing.Optional[int]. Number of tokens to return in a page (default 10)

            - page_number: typing.Optional[int]. Based on pageSize, which page of records to return
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"users/{user_id}/api-token"),
            params=remove_none_from_dict({"tenantId": tenant_id, "pageSize": page_size, "pageNumber": page_number}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListApiTokensResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_api_token(self, user_id: UserId, *, tenant_id: str) -> ApiTokenResponse:
        """
        Creates an api token for authenticating against Flatfile APIs.

        Parameters:
            - user_id: UserId. The user id

            - tenant_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"users/{user_id}/api-token"),
            params=remove_none_from_dict({"tenantId": tenant_id}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ApiTokenResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def exchange_token(
        self,
        *,
        token: typing.Optional[str] = OMIT,
        email: typing.Optional[str] = OMIT,
        space_id: typing.Optional[str] = OMIT,
    ) -> ExchangeTokenResponse:
        """
        Exchange an invitation for an access token

        Parameters:
            - token: typing.Optional[str]. The token to use for the request

            - email: typing.Optional[str]. The invited guest's email, required if no token

            - space_id: typing.Optional[str]. The spaceId, required if no token
        """
        _request: typing.Dict[str, typing.Any] = {}
        if token is not OMIT:
            _request["token"] = token
        if email is not OMIT:
            _request["email"] = email
        if space_id is not OMIT:
            _request["spaceId"] = space_id
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "invitations/exchange"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ExchangeTokenResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
